/* Code Time Technologies 2017																		*/

/* ------------------------------------------------------------------------------------------------ */
/* main() / Adam & Eve stack size.																	*/
/* NOTE: The heap will use all memory left in RAM0													*/
/*       The stack (A&E stack) is located at the end of RAM0										*/

_STACK_SIZE = DEFINED(_STACK_SIZE) ? _STACK_SIZE : 32768;

/* ------------------------------------------------------------------------------------------------ */
/* Memory reserved for the M-drive																	*/

_MDRV_SIZE				= DEFINED(_MDRV_SIZE)          ? _MDRV_SIZE          : 0x10000000;

/* ------------------------------------------------------------------------------------------------ */

MEMORY
{
   RAM0 : ORIGIN = 0x000000000, LENGTH = 0x80000000
   OCM  : ORIGIN = 0x0FFFC0000, LENGTH = 0x00040000
   QSPI : ORIGIN = 0x0C0000000, LENGTH = 0x20000000
}

/* Specify the default entry point to the program */

ENTRY(_vector_table)

/* Define the sections, and where they are mapped in memory */

SECTIONS
{
	.text : {
    	CREATE_OBJECT_SYMBOLS
		_region_start_ram = .;
		KEEP (*(.vectors))
		*(.boot)
		*(.text)
		*(.text.*)
		*(.gnu.linkonce.t.*)
		*(.plt)
		*(.gnu_warning)
		*(.gcc_execpt_table)
		*(.glue_7)
		*(.glue_7t)
		*(.ARM.extab)
		*(.gnu.linkonce.armextab.*)
	} > RAM0

	.init (ALIGN(64)) : {
		KEEP (*(.init))
	} > RAM0

	.fini (ALIGN(64)) : {
		KEEP (*(.fini))
	} > RAM0

	.interp : {
		KEEP (*(.interp))
	} > RAM0

	.note-ABI-tag : {
		KEEP (*(.note-ABI-tag))
	} > RAM0

	.rodata : {
		. = ALIGN(64);
		__rodata_start = .;
		*(.rodata)
		*(.rodata.*)
		*(.gnu.linkonce.r.*)
		__rodata_end = .;
	} > RAM0

	.rodata1 : {
		. = ALIGN(64);
		__rodata1_start = .;
		*(.rodata1)
		*(.rodata1.*)
		__rodata1_end = .;
	} > RAM0

	.sdata2 : {
		. = ALIGN(64);
		__sdata2_start = .;
		*(.sdata2)
		*(.sdata2.*)
		*(.gnu.linkonce.s2.*)
		__sdata2_end = .;
	} > RAM0

	.sbss2 : {
		. = ALIGN(64);
		__sbss2_start = .;
		*(.sbss2)
		*(.sbss2.*)
		*(.gnu.linkonce.sb2.*)
		__sbss2_end = .;
	} > RAM0

	.data : {
		. = ALIGN(64);
		__data_start = .;
		*(.data)
		*(.data.*)
		*(.gnu.linkonce.d.*)
		*(.jcr)
		*(.got)
		*(.got.plt)
		__data_end = .;
	} > RAM0

	.data1 : {
		. = ALIGN(64);
		__data1_start = .;
		*(.data1)
		*(.data1.*)
		__data1_end = .;
	} > RAM0

	.got : {
		*(.got)
	} > RAM0

	.got1 : {
		*(.got1)
	} > RAM0

	.got2 : {
		*(.got2)
	} > RAM0

	.ctors : {
		. = ALIGN(64);
		__CTOR_LIST__ = .;
		___CTORS_LIST___ = .;
		KEEP (*crtbegin.o(.ctors))
		KEEP (*(EXCLUDE_FILE(*crtend.o) .ctors))
		KEEP (*(SORT(.ctors.*)))
		KEEP (*(.ctors))
		__CTOR_END__ = .;
		___CTORS_END___ = .;
	} > RAM0

	.dtors : {
		. = ALIGN(64);
		__DTOR_LIST__ = .;
		___DTORS_LIST___ = .;
		KEEP (*crtbegin.o(.dtors))
		KEEP (*(EXCLUDE_FILE(*crtend.o) .dtors))
		KEEP (*(SORT(.dtors.*)))
		KEEP (*(.dtors))
		__DTOR_END__ = .;
		___DTORS_END___ = .;
	} > RAM0

	.fixup : {
		__fixup_start = .;
		*(.fixup)
		__fixup_end = .;
	} > RAM0

	.eh_frame : {
		*(.eh_frame)
	} > RAM0

	.eh_framehdr : {
		__eh_framehdr_start = .;
		*(.eh_framehdr)
		__eh_framehdr_end = .;
	} > RAM0

	.gcc_except_table : {
		*(.gcc_except_table)
	} > RAM0

	.mmu_tbl (ALIGN(4096)) : {
		__mmu_tbl_start = .;
		*(.mmu_tbl)
		__mmu_tbl_end = .;
	} > RAM0

	.mmu_tbl_1 (ALIGN(4096)) : {
		__mmu_tbl_1_start = .;
		*(.mmu_tbl_1)
		__mmu_tbl_1_end = .;
	} > RAM0

	.mmu_tbl_2 (ALIGN(4096)) : {
		__mmu_tbl_2_start = .;
		*(.mmu_tbl_2)
		__mmu_tbl_2_end = .;
	} > RAM0

	.mmu_tbl_3 (ALIGN(4096)) : {
		__mmu_tbl_3_start = .;
		*(.mmu_tbl_3)
		__mmu_tbl_3_end = .;
	} > RAM0

	.ARM.exidx : {
		__exidx_start = .;
		*(.ARM.exidx*)
		*(.gnu.linkonce.armexidix.*.*)
		__exidx_end = .;
	} > RAM0

	.preinit_array : {
		. = ALIGN(64);
		__preinit_array_start = .;
		KEEP (*(SORT(.preinit_array.*)))
		KEEP (*(.preinit_array))
		__preinit_array_end = .;
	} > RAM0

	.init_array : {
		. = ALIGN(64);
		__init_array_start = .;
		KEEP (*(SORT(.init_array.*)))
		KEEP (*(.init_array))
		__init_array_end = .;
	} > RAM0

	.fini_array : {
		. = ALIGN(64);
		__fini_array_start = .;
		KEEP (*(SORT(.fini_array.*)))
		KEEP (*(.fini_array))
		__fini_array_end = .;
	} > RAM0

	.ARM.attributes : {
		__ARM.attributes_start = .;
		*(.ARM.attributes)
		__ARM.attributes_end = .;
	} > RAM0

	.sdata : {
		. = ALIGN(64);
		__sdata_start = .;
		*(.sdata)
		*(.sdata.*)
		*(.gnu.linkonce.s.*)
		__sdata_end = .;
	} > RAM0

	.sbss (NOLOAD) : {
		. = ALIGN(64);
		__sbss_start = .;
		*(.sbss)
		*(.sbss.*)
		*(.gnu.linkonce.sb.*)
		. = ALIGN(64);
		__sbss_end = .;
	} > RAM0

	.tdata : {
		. = ALIGN(64);
		__tdata_start = .;
		*(.tdata)
		*(.tdata.*)
		*(.gnu.linkonce.td.*)
		__tdata_end = .;
	} > RAM0

	.tbss : {
		. = ALIGN(64);
		__tbss_start = .;
		*(.tbss)
		*(.tbss.*)
		*(.gnu.linkonce.tb.*)
		__tbss_end = .;
	} > RAM0

	.bss (NOLOAD) :
	{
		. = ALIGN(64);
		__bss_start__ = .;
		*(.bss)
		*(.bss.*)
		*(.gnu.linkonce.b.*)
		*(COMMON)
		. = ALIGN(64);
		__bss_end__ = .;
	} > RAM0

	_SDA_BASE_  = __sdata_start  + ((__sbss_end  - __sdata_start)  / 2 );
	_SDA2_BASE_ = __sdata2_start + ((__sbss2_end - __sdata2_start) / 2 );

	.memdrive (NOLOAD) : {
		. = ALIGN(8);
		G_MemDrvBase = .;
		. += _MDRV_SIZE;
		G_MemDrvEnd = .;
	} > RAM0

	. = ALIGN(64);							/* Outside .heap & .stack								*/
											/* Both .heap & .stack have size of N*64				*/
	.heap (NOLOAD):
	{
		_heap = .;
		HeapBase = .;
		_heap_start = .;
		. = (_region_start_ram + ((LENGTH(RAM0) + 63) & 0xFFFFFFFFFFFFFFC0))
		  - ((_STACK_SIZE                       + 63) & 0xFFFFFFFFFFFFFFC0);
		_heap_end = .;
		HeapLimit = .;
	} > RAM0

	.stack (NOLOAD):
	{
		_stack_end = .;
		. += ((_STACK_SIZE + 63) & 0xFFFFFFFFFFFFFFC0);
		_stack       = .;
		_stack_start = .;
	} > RAM0

	_end = .;

	.uncached (NOLOAD) : ALIGN (64)
	{
		*(.uncached)
		*(.uncached.*)
  	} >OCM
}

/* EOF */
